/*
 * Copyright 2007 Niels Provos <provos@citi.umich.edu>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/types.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <sys/queue.h>

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <err.h>
#include <errno.h>

#include <event.h>
#include <evhttp.h>

#include "dnsdata.gen.h"

int
web_fill_in_data(int trunc_body_to_len,
    struct evhttp_request *request, struct HTTPServerReply *reply)
{
	struct evkeyval *kv;

	if (EVTAG_ASSIGN(reply, response_code, request->response_code) == -1)
		return (-1);

	if (EVTAG_ASSIGN(reply, response_string,
		request->response_code_line) == -1)
		return (-1);

	TAILQ_FOREACH(kv, request->input_headers, next) {
		struct HTTPHeader *header = EVTAG_ADD(reply, headers);
		if (header == NULL)
			return (-1);

		if (EVTAG_ASSIGN(header, key, kv->key) == -1)
			return (-1);

		if (EVTAG_ASSIGN(header, value, kv->value) == -1)
			return (-1);
	}

	if (EVBUFFER_LENGTH(request->input_buffer)) {
		int len = EVBUFFER_LENGTH(request->input_buffer);
		if (trunc_body_to_len && trunc_body_to_len < len)
			len = trunc_body_to_len;
		if (EVTAG_ASSIGN(reply, body_content, 
			EVBUFFER_DATA(request->input_buffer), len) == -1)
			return (-1);
	}

	return (0);
}
